@@ -8,15 +8,16 @@
 # @author Davide Brunato <brunato@sissa.it>
 #
 from abc import abstractmethod
+import re
 import math
 import operator
-import re
 import datetime
 from calendar import isleap
 from decimal import Decimal, Context
-from typing import cast, Any, Callable, Dict, Optional, Tuple, Union
+from typing import cast, Any, Dict, Optional, Tuple, Type, TypeVar, Union
 
-from ..helpers import MONTH_DAYS_LEAP, MONTH_DAYS, DAYS_IN_4Y, \
+from elementpath._typing import Callable
+from elementpath.helpers import MONTH_DAYS_LEAP, MONTH_DAYS, DAYS_IN_4Y, \
     DAYS_IN_100Y, DAYS_IN_400Y, days_from_common_era, adjust_day, \
     normalized_seconds, months2days, round_number
 from .atomic_types import AnyAtomicType
@@ -112,6 +113,9 @@ class Timezone(datetime.tzinfo):
         raise TypeError("fromutc() argument must be a datetime.datetime instance")
 
 
+_DT = TypeVar('_DT', bound='AbstractDateTime')
+
+
 class AbstractDateTime(AnyAtomicType):
     """
     A class for representing XSD date/time objects. It uses and internal datetime.datetime
@@ -171,7 +175,31 @@ class AbstractDateTime(AnyAtomicType):
 
     @abstractmethod
     def __str__(self) -> str:
-        raise NotImplementedError()
+        raise NotImplementedError
+
+    @abstractmethod
+    def __lt__(self, other: object) -> bool:
+        raise NotImplementedError
+
+    @abstractmethod
+    def __le__(self, other: object) -> bool:
+        raise NotImplementedError
+
+    @abstractmethod
+    def __gt__(self, other: object) -> bool:
+        raise NotImplementedError
+
+    @abstractmethod
+    def __ge__(self, other: object) -> bool:
+        raise NotImplementedError
+
+    @abstractmethod
+    def __add__(self, other: object) -> Any:
+        raise NotImplementedError
+
+    @abstractmethod
+    def __sub__(self, other: object) -> Any:
+        raise NotImplementedError
 
     @property
     def year(self) -> int:
@@ -236,8 +264,8 @@ class AbstractDateTime(AnyAtomicType):
         return self._dt.isocalendar()
 
     @classmethod
-    def fromstring(cls, datetime_string: str, tzinfo: Optional[Timezone] = None) \
-            -> 'AbstractDateTime':
+    def fromstring(cls: Type[_DT], datetime_string: str, tzinfo: Optional[Timezone] = None) \
+            -> _DT:
         """
         Creates an XSD date/time instance from a string formatted value.
 
@@ -287,13 +315,13 @@ class AbstractDateTime(AnyAtomicType):
         return cls(tzinfo=tzinfo, **kwargs)
 
     @classmethod
-    def fromdatetime(cls, dt: Union[datetime.datetime, datetime.date, datetime.time],
-                     year: Optional[int] = None) -> 'AbstractDateTime':
+    def fromdatetime(cls: Type[_DT], dt: Union[datetime.datetime, datetime.date, datetime.time],
+                     year: Optional[int] = None) -> _DT:
         """
         Creates an XSD date/time instance from a datetime.datetime/date/time instance.
 
         :param dt: the datetime, date or time instance that stores the XSD Date/Time value.
-        :param year: if an year is provided the created instance refers to it and the \
+        :param year: if a year is provided the created instance refers to it and the \
         possibly present *dt.year* part is ignored.
         :return: an AbstractDateTime concrete subclass instance.
         """
@@ -307,7 +335,7 @@ class AbstractDateTime(AnyAtomicType):
             kwargs['year'] = year
         return cls(**kwargs)
 
-    # Python can't compares offset-naive and offset-aware datetimes
+    # Python can't compare offset-naive and offset-aware datetimes
     def _get_operands(self, other: object) -> Tuple[datetime.datetime, datetime.datetime]:
         if isinstance(other, (self.__class__, datetime.datetime)) or \
                 isinstance(self, other.__class__):
@@ -348,7 +376,7 @@ class OrderedDateTime(AbstractDateTime):
 
     @abstractmethod
     def __str__(self) -> str:
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @classmethod
     def fromdelta(cls, delta: datetime.timedelta, adjust_timezone: bool = False) \
@@ -731,6 +759,9 @@ class Time(AbstractDateTime):
             raise TypeError("wrong type %r for operand %r" % (type(other), other))
 
 
+_D = TypeVar('_D', bound='Duration')
+
+
 class Duration(AnyAtomicType):
     """
     Base class for the XSD duration types.
@@ -793,7 +824,7 @@ class Duration(AnyAtomicType):
         return value
 
     @classmethod
-    def fromstring(cls, text: str) -> 'Duration':
+    def fromstring(cls: Type[_D], text: str) -> _D:
         """
         Creates a Duration instance from a formatted XSD duration string.
 
